--[[
*************************************************************************
    SLTimex: Core.lua
    Author: Taffu of Draka
    Version: 1.0
    Website: http://www.wowinterface.com/portal.php?&uid=24019
    Revision Date: October 11, 2007
    
    This file is part of SLTimex.

    SLTimex is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    SLTimex is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SLTimex.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************
]]

local L = AceLibrary("AceLocale-2.2"):new("SLT")
local D = AceLibrary("Dewdrop-2.0")

local opts = { 
    type='group',
    args = {
        header = {
            type = 'header',
            name = L["SLTimex"],
            order = 1,
        },
        lock = {
            type = 'toggle',
            name = L["Lock"],
            desc = L["Locks the frame in place."],
            get = "GetLocked",
            set = "SetLocked",
            order = 2,
        },
        display = {
            name = L["Display"],
            desc = L["Display Styles."],
            type = 'group',
            order = 3,
            args = {
                bottomclock = {
                    type = 'toggle',
                    name = L["Bottom Clock"],
                    desc = L["Toggles the clock position to the bottom."],
                    get = "IsClockBottom",
                    set = "ToggleClockBottom",
                },
                justify = {
                    type = 'text',
                    name = L["Justify Text"],
                    desc = L["Sets the text justification for the display."],
                    usage = L["<LEFT, RIGHT, or CENTER>"],
                    validate = { L["LEFT"], L["RIGHT"], L["CENTER"] },
                    get = "GetTextJustify",
                    set = "SetTextJustify",
                },
                clock24Hour = {
                    type = 'toggle',
                    name = L["24 Hour Clock"],
                    desc = L["Toggles between 24 Hour and 12 Hour style clock."],
                    get = "Is24Hour",
                    set = "Toggle24Hour",
                },
                setfont = {
                    type = 'text',
                    name = L["Font"],
                    desc = L["Choose available Font Style."],
                    usage = L["<Font Name>"],
                    validate = { 
                        "Adventure", 
                        "Bazooka", 
                        "BigNoodle", 
                        "BlackChancery", 
                        "Carolingia", 
                        "Diablo", 
                        "Digital", 
                        "Emblem", 
                        "Enigmatic", 
                        "Hurtmold", 
                        "JMADH", 
                        "Morpheus", 
                        "MoviePoster", 
                        "Porky", 
                        "Prozak", 
                        "Quake", 
                        "Stonehenge", 
                        "Timeless", 
                        "Tw Cen MT", 
                        "Yellowjacket" 
                    },
                    get = "GetCurFont",
                    set = "SetCurFont",
                },
            },
        },
        seperator = {
            name = L["Seperator"],
            desc = L["Seperator Styles."],
            type = 'group',
            order = 4,
            args = {
                stats = {
                    type = 'text',
                    name = L["Stats"],
                    desc = L["Sets the character used to seperate the stats text line."],
                    usage = L["<Seperator Character>"],
                    get = "GetStatSep",
                    set = "SetStatSep",
                },
                clock = {
                    type = 'text',
                    name = L["Clock"],
                    desc = L["Sets the character used to seperate Hour and Minute on the clock."],
                    usage = L["<Seperator Character>"],
                    get = "GetClockSep",
                    set = "SetClockSep",
                },
            },
        },
        modules = {
            name = L["Modules"],
            desc = L["Stats Modules."],
            type = 'group',
            order = 5,
            args = {
                toggleMemory = {
                    type = 'toggle',
                    name = L["Toggle Memory"],
                    desc = L["Toggles the memory stat on and off."],
                    get = "IsMemShown",
                    set = "ToggleMemShown",
                },
                toggleLatency = {
                    type = 'toggle',
                    name = L["Toggle Latency"],
                    desc = L["Toggles the latency stat on and off."],
                    get = "IsLagShown",
                    set = "ToggleLagShown",
                },
                toggleFPS = {
                    type = 'toggle',
                    name = L["Toggle FPS"],
                    desc = L["Toggles the FPS stat on and off."],
                    get = "IsFPSShown",
                    set = "ToggleFPSShown",
                },
            },
        },
        scale = {
            type = 'range',
            name = L["Set Scale"],
            desc = L["Sets the scale."],
            usage = L["<Range 0.1 to 2.0>"],
            step = 0.1,
            min = 0.1,
            max = 2.0,
            get = "GetScale",
            set = "SetScale",
            order = 6,
        },
        offsetX = {
            type = 'range',
            name = L["Offset X"],
            desc = L["Sets the horizontal frame position."],
            usage = L["<Range -1000 to 1000>"],
            step = 1,
            min = -1000,
            max = 1000,
            get = "GetOffsetX",
            set = "SetOffsetX",
            order = 7,
        },
        offsetY = {
            type = 'range',
            name = L["Offset Y"],
            desc = L["Sets the vertical frame position."],
            usage = L["<Range -1000 to 1000>"],
            step = 1,
            min = -1000,
            max = 1000,
            get = "GetOffsetY",
            set = "SetOffsetY",
            order = 8,
        },
    },
}

SLT = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")

function SLT:OnInitialize()
    SLT:RegisterChatCommand(L["Slash-Commands"], opts)
    SLT:RegisterDB("SLTVar", "SLTVarChar")
    SLT:RegisterDefaults("profile", {
        textJustH = "RIGHT",
        frameOffsetX = 150,
        frameOffsetY = 150,
        statSep = "+",
        clockSep = ":",
        frameScale = 1.0,
        frameLock = false,
        hour24Clock = false,
        showFPS = true,
        showMEM = true,
        showLAG = true,
        clockBottom = false,
        curFont = "Timeless",
    })

    self:CreateSLTFrame()
end

function SLT:OnEnable()
    SLTFrame:Show()
    self:ScheduleRepeatingEvent(self.UpdateFrame, 1, self)
end

function SLT:OnDisable()
    SLTFrame:Hide()
end

function SLT:CreateSLTFrame()
    local frame = CreateFrame("Frame", "SLTFrame", UIParent)
    frame:SetBackdrop({
        bgFile = "Interface/Tooltips/UI-Tooltip-Background",
        tile = true, tileSize = 16,
        insets = { left = 0, top = 0, right = 0, bottom = 0 }
    })
    frame:SetBackdropColor(0, 0, 0, 0)
    frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
    frame:SetWidth(220)
    frame:SetHeight(64)
    frame:SetFrameStrata("BACKGROUND")
    frame:SetScript("OnDragStart", 
        function()
            if not self.db.profile.frameLock then 
                this:StartMoving() 
            end
        end)
	frame:SetScript("OnDragStop",
		function()
			this:StopMovingOrSizing()
			local p, rt, rp, x, y = this:GetPoint()
			self.db.profile.frameOffsetX = math.floor(x)
			self.db.profile.frameOffsetY = math.floor(y)
        end)
    frame:SetMovable(true)
	frame:RegisterForDrag("LeftButton")
    frame:EnableMouse(true)
    
    D:Register(frame, 'children', function() D:FeedAceOptionsTable(opts) end, 'cursorX', true, 'cursorY', true)
    
    frame.time = frame:CreateFontString("SLTTime", "OVERLAY")
    frame.time:SetJustifyH("Right")
    frame.time:SetShadowColor(0, 0, 0, 0.8)
    frame.time:SetShadowOffset(1, -1)
    
    frame.stats = frame:CreateFontString("SLTStats", "OVERLAY")
    frame.stats:SetJustifyH("Right")
    frame.stats:SetShadowColor(0, 0, 0, 0.8)
    frame.stats:SetShadowOffset(1, -1)
end

function SLT:UpdateFrame()
    SLTFrame:SetPoint("CENTER", UIParent, "CENTER", self.db.profile.frameOffsetX, self.db.profile.frameOffsetY)
    SLTFrame:SetScale(self.db.profile.frameScale)
    
    SLTTime:SetJustifyH(self.db.profile.textJustH)
    SLTStats:SetJustifyH(self.db.profile.textJustH)
    
    if self.db.profile.curFont == "BlackChancery" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\BlackChancery.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\BlackChancery.ttf", 14)
    elseif self.db.profile.curFont == "Adventure" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Adventure.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Adventure.ttf", 14)
    elseif self.db.profile.curFont == "Timeless" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Timeless.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Timeless.ttf", 14)
    elseif self.db.profile.curFont == "Porky" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Porky.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Porky.ttf", 14)
    elseif self.db.profile.curFont == "Yellowjacket" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Yellowjacket.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Yellowjacket.ttf", 14)
    elseif self.db.profile.curFont == "Emblem" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Emblem.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Emblem.ttf", 14)
    elseif self.db.profile.curFont == "BigNoodle" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\BigNoodle.ttf", 28)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\BigNoodle.ttf", 18)
    elseif self.db.profile.curFont == "Diablo" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Diablo.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Diablo.ttf", 14)
    elseif self.db.profile.curFont == "MoviePoster" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\MoviePoster.ttf", 28)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\MoviePoster.ttf", 18)
    elseif self.db.profile.curFont == "Bazooka" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Bazooka.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Bazooka.ttf", 14)
    elseif self.db.profile.curFont == "Enigmatic" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Enigmatic.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Enigmatic.ttf", 14)
    elseif self.db.profile.curFont == "Tw Cen MT" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Tw_Cen_MT.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Tw_Cen_MT.ttf", 14)
    elseif self.db.profile.curFont == "Carolingia" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Carolingia.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Carolingia.ttf", 14)
    elseif self.db.profile.curFont == "Quake" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Quake.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Quake.ttf", 14)
    elseif self.db.profile.curFont == "Digital" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Digital.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Digital.ttf", 14)
    elseif self.db.profile.curFont == "Morpheus" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\MORPHEUS.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\MORPHEUS.ttf", 14)
    elseif self.db.profile.curFont == "Prozak" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Prozak.ttf", 26)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Prozak.ttf", 16)
    elseif self.db.profile.curFont == "Stonehenge" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Stonehenge.ttf", 24)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Stonehenge.ttf", 14)
    elseif self.db.profile.curFont == "Hurtmold" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Hurtmold.ttf", 26)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\Hurtmold.ttf", 16)
    elseif self.db.profile.curFont == "JMADH" then
        SLTTime:SetFont("Interface\\Addons\\SLTimex\\Fonts\\JMADH.ttf", 30)
        SLTStats:SetFont("Interface\\Addons\\SLTimex\\Fonts\\JMADH.ttf", 20)
    end
    
    if self.db.profile.clockBottom then
        SLTStats:SetPoint("TOP", "SLTFrame", "TOP", 0, 0)
        SLTTime:SetPoint("TOP", "SLTStats", "BOTTOM", 0, 0)
    else
        SLTTime:SetPoint("TOP", "SLTFrame", "TOP", 0, 0)
        SLTStats:SetPoint("TOP", "SLTTime", "BOTTOM", 0, 0)
    end
    
    local hour = date("%H")
    local minute = date("%M")
    local clockSeperator = self.db.profile.clockSep
    
    if self.db.profile.hour24Clock then
        SLTTime:SetText(hour..clockSeperator..minute)
    else
        local fixHour = math.floor(hour-hour)+date("%H")

        if (fixHour > 11 and fixHour ~= 24) then
            newHour = math.floor(fixHour-12)
            SLTTime:SetText(newHour..clockSeperator..minute.."pm")
        else
            newHour = fixHour
            SLTTime:SetText(newHour..clockSeperator..minute.."am")
        end
    end
    
    local down, up, lag = GetNetStats()
    local fps = GetFramerate()
    local mem = floor(collectgarbage("count") / 1024)
    local statSep = self.db.profile.statSep
    
    if self.db.profile.showMEM and self.db.profile.showLAG and self.db.profile.showFPS then
        SLTStats:SetText(mem.."mb|cffffea00"..statSep.."|r"..lag.."ms|cffffea00"..statSep.."|r"..floor(fps).."fps") -- All three showing
    elseif not self.db.profile.showMEM and self.db.profile.showLAG and self.db.profile.showFPS then
        SLTStats:SetText(lag.."ms|cffffea00"..statSep.."|r"..floor(fps).."fps") -- LAG & FPS showing
    elseif self.db.profile.showMEM and not self.db.profile.showLAG and self.db.profile.showFPS then
        SLTStats:SetText(mem.."mb|cffffea00"..statSep.."|r"..floor(fps).."fps") -- MEM & FPS showing
    elseif self.db.profile.showMEM and self.db.profile.showLAG and not self.db.profile.showFPS then
        SLTStats:SetText(mem.."mb|cffffea00"..statSep.."|r"..lag.."ms") -- MEM & LAG showing
    elseif not self.db.profile.showMEM and not self.db.profile.showLAG and self.db.profile.showFPS then
        SLTStats:SetText(floor(fps).."fps") -- Just FPS showing
    elseif not self.db.profile.showMEM and self.db.profile.showLAG and not self.db.profile.showFPS then
        SLTStats:SetText(lag.."ms") -- Just LAG showing
    elseif self.db.profile.showMEM and not self.db.profile.showLAG and not self.db.profile.showFPS then
        SLTStats:SetText(mem.."mb") -- Just MEM showing
    elseif not self.db.profile.showMEM and not self.db.profile.showLAG and not self.db.profile.showFPS then
        SLTStats:SetText("") -- Nothing showing
    end
    
    SLTFrame:SetHeight(SLTTime:GetHeight() + SLTStats:GetHeight())
    
    -- By limitation the actual frame & text frames should never be greater than 220px wide
    -- This avoids the text/frame jumping around as text values change so the addon stays still
    SLTFrame:SetWidth(220)
    SLTTime:SetWidth(220)
    SLTStats:SetWidth(220)
end

function SLT:ShowOptions()
	D:Open( UIParent, 'children', function() D:FeedAceOptionsTable(opts) end, 'cursorX', true, 'cursorY', true ) 
end

-- Options stuff
function SLT:IsMemShown()
    return self.db.profile.showMEM
end

function SLT:ToggleMemShown()
    self.db.profile.showMEM = not self.db.profile.showMEM
    self:UpdateFrame()
end

function SLT:IsLagShown()
    return self.db.profile.showLAG
end

function SLT:ToggleLagShown()
    self.db.profile.showLAG = not self.db.profile.showLAG
    self:UpdateFrame()
end

function SLT:IsFPSShown()
    return self.db.profile.showFPS
end

function SLT:ToggleFPSShown()
    self.db.profile.showFPS = not self.db.profile.showFPS
    self:UpdateFrame()
end

function SLT:GetLocked()
    return self.db.profile.frameLock
end

function SLT:SetLocked()
    self.db.profile.frameLock = not self.db.profile.frameLock
    self:UpdateFrame()
end

function SLT:Is24Hour()
    return self.db.profile.hour24Clock
end

function SLT:Toggle24Hour()
    self.db.profile.hour24Clock = not self.db.profile.hour24Clock
    self:UpdateFrame()
end

function SLT:GetScale()
    return self.db.profile.frameScale
end

function SLT:SetScale(newValue)
    self.db.profile.frameScale = newValue
    self:UpdateFrame()
end

function SLT:GetStatSep()
    return self.db.profile.statSep
end

function SLT:SetStatSep(newValue)
    self.db.profile.statSep = newValue
    self:UpdateFrame()
end

function SLT:GetClockSep()
    return self.db.profile.clockSep
end

function SLT:SetClockSep(newValue)
    self.db.profile.clockSep = newValue
    self:UpdateFrame()
end

function SLT:GetTextJustify()
    return self.db.profile.textJustH
end

function SLT:SetTextJustify(newValue)
    self.db.profile.textJustH = newValue
    self:UpdateFrame()
end

function SLT:GetOffsetX()
    return self.db.profile.frameOffsetX
end

function SLT:SetOffsetX(newValue)
    self.db.profile.frameOffsetX = newValue
    self:UpdateFrame()
end

function SLT:GetOffsetY()
    return self.db.profile.frameOffsetY
end

function SLT:SetOffsetY(newValue)
    self.db.profile.frameOffsetY = newValue
    self:UpdateFrame()
end

function SLT:IsClockBottom()
    return self.db.profile.clockBottom
end

function SLT:ToggleClockBottom()
    self.db.profile.clockBottom = not self.db.profile.clockBottom
    self:UpdateFrame()
end

function SLT:GetCurFont()
    return self.db.profile.curFont
end

function SLT:SetCurFont(newValue)
    self.db.profile.curFont = newValue
    self:UpdateFrame()
end